function DMC_Iterativo_siso()
clear all; clc
%==========================================================================
%            Obtencin de los coeficientes Su de la rta al escaln
%==========================================================================
Ts=4;%tiempo de muestreo
sis=c2d(tf([0 4.05],[50 1]),Ts,'zoh');%sistema
[numsis densis]=tfdata(sis,'v');
Su=step(sis);
Su=Su(2:end);%vector de coeficientes de respuesta al escaln
N=length(Su);%longitud del vector de coeficientes Su
%==========================================================================
%                               Parmetros
%==========================================================================
instantes=50;%nmero de periodos de muestreo que se ejecutar el algoritmo DMC
Nc=5; %horizonte de control
Np=N+Nc;%horizonte de prediccin
alfa=1;%parmetros alfa lambda y W
lambda=1;
ymin=-0.15;%lmites mximos y mnimos de las VC
ymax= 0.15;
umin=-0.05;%lmites mximos y mnimos y de incrementos de las VM
umax= 0.05;
deltaumin=-0.01;
deltaumax= 0.01;
r=-0.2;%referencia 
%==========================================================================
%                    Parmetros del algoritmo recursivo
%==========================================================================
n=500;%nmero de veces que se ejecuta el algoritmo recursivo para encontrar solucin
tol1=0.01;%tolerancias de los lmites de las entradas, incrementos y salidas 
tol2=0.009;
tol3=0.01;
w1=5;
w2=5;
w3=5;
%==========================================================================
%                Clculo de las matrices G, Q1, Q2, alfa, lambda y M
%==========================================================================
if Np>N;
    Saux=[Su; ones(Np-N,1)*Su(end)];
else
    Saux=Su;
end
G=zeros(Np,Nc);
for i=1:Nc;
    G(i:end,i)=Saux(1:Np-i+1);
end
Q1=zeros(N,N);
Q1(1:end-1,2:end)=eye(N-1,N-1);
Q1(end,end)=1;
Q2=zeros(Np,N);
if Np==N;
    Q2=Q1;
elseif Np<N
    Q2(1:end,2:Np+1)=eye(Np,Np);
else
    Q2(1:N-1,2:end)=eye(N-1,N-1);
    Q2(N:end,end)=1;
end
alfa=alfa*eye(Np,Np);
lambda=lambda*eye(Nc,Nc);
M=inv(G'*alfa*G+lambda)*G'*alfa;
%==========================================================================
%                   matrices del algoritmo recursivo
%==========================================================================
Pm0=inv(G'*alfa*G+lambda);
%==========================================================================
%                    condiciones iniciales
%==========================================================================
t=(0:1:instantes);%vector de tiempo total en instantes
R=r*ones(instantes+Np,1);%vector de trayectoria de referencia
ymvec=0;%Vector que contendra el historial de sallidas medidas
ypvec=ymvec;%vector que contendra el historial de predicciones para cada instante (k+1|k)
deltauvec=0;%vector que contendra el historial de incrementos deltau para cada instante (k|k)
uvec=deltauvec;%vector que contendra el historial de la entrada para cada instante (k|k)
zyu=0;%cond. iniciales para poder simular los sistemas mediante el comando filter
L=ymvec*ones(N,1);%inicializacion de la resuesta libre
%==========================================================================
%                       Ejecucin del algoritmo 
%==========================================================================
for i=1:instantes;
    L=Q1*L+Su*deltauvec(end);%actualizacin de la rta libre
    w=(ymvec(end)-L(1))*ones(Np,1);%estimacin de la perturbacin no medible
    T=Q2*L+w;
    D=R(i:i+Np-1)-T;
    deltau=M*D;%clculo de los movimientos futuros
    %====================================================================================================
    % aplicacin del algoritmo recursivo
    %================================================================================================           
    cont=0;
    Pm=Pm0;
    while (cont<n)
        yp=T+G*deltau;%salida predicha
        ufut=uvec(end)+cumsum(deltau);
        flagu=verificar(ufut,umax,umin);
        flagdeltau=verificar(deltau,deltaumax,deltaumin);
        flagyp=verificar(yp,ymax,ymin);
        if  0==sum([flagu flagdeltau flagyp])
            break
        end
        cont=cont+1;  
        E=0;
        h=zeros(1,Nc);
        if flagdeltau>0
            [E1,h1]=deltau_rest(flagdeltau,deltau,deltaumax,deltaumin,w1,tol1);
            E=[E;E1];
            h=[h;h1];
        end
        if flagu>0
            [E2,h2]=u_rest(flagu,ufut,uvec(end),umax,umin,w2,tol2);
            E=[E;E2];
            h=[h;h2];
        end
        if flagyp>0
            [E3,h3]=yp_rest(flagyp,yp,ymax,ymin,w3,tol3,G,T);
            E=[E;E3];
            h=[h;h3];
        end
        E=E(2:end);
        h=h(2:end,:)';
        m1=h'*Pm;
        m2=Pm*h*inv(m1*h+eye(size(h,2)));
        deltau=deltau+m2*(E-h'*deltau);
        Pm=Pm-m2*m1;
    end
    yp=T+G*deltau;%salida predicha
    ypvec(i+1)=yp(1);%salida predicha del instante Yp(k+1|k)
    deltauvec(i+1)=deltau(1);%deltau del instante deltau(k|k) 
    uvec(i+1)=uvec(end)+deltau(1);%accion de control del instante u(k|k)
    %======================================================================
    %       aplicacion de los incrementos y medicin de la salida
    %======================================================================    
    [y,zyu]=filter(numsis(2:end),densis,uvec(end),zyu);%se aplica la accin de control
    ymvec(i+1)=y;%se mide la salida del proceso
    %======================================================================
    %======================================================================
end
R=R(1:instantes+1);
deltauvec=deltauvec(2:end);
uvec=uvec(2:end);
%==========================================================================
%                        Grficas de las variables
%==========================================================================
figure(1)
plot(t,ymvec,'b');
hold on
stairs(t,ypvec,'r');
stairs(t,R,'--g');
hold off
title('Salida')
xlabel('instantes')
legend('Yreal','Ypredicha','Ref')

figure(2)
subplot(211)
stairs(t(1:end-1),uvec,'b');
title('Acciones de control')
xlabel('instantes')
legend('u')
subplot(212)
stairs(t(1:end-1),deltauvec,'b');
title('Incrementos en las acciones de control')
xlabel('instantes')
legend('deltau')
%==========================================================================
%verificar si los incrementos, entradas o predicciones violan restricciones
%==========================================================================
function flag=verificar(vector,lim_max,lim_min)
flag=0;
if min(vector)<lim_min
   flag=flag+1;
end
if max(vector)>lim_max
   flag=flag+2;
end
%==========================================================================
% Clculo de las matrices de restricciones h1 y E1 para los incrementos
%==========================================================================
function  [E,h]=deltau_rest(flag,deltau,deltau_max,deltau_min,W,tol)
h=zeros(1,length(deltau));
E=0;
switch flag
    case 1
        for j=1:length(deltau);
            if  deltau(j)<deltau_min;
                h(end+1,j)=W;
                E(end+1,1)=(deltau_min+tol)*W; 
            end   
        end
    case 2
        for j=1:length(deltau);
            if  deltau(j)>deltau_max;
                h(end+1,j)=W;
                E(end+1,1)=(deltau_max-tol)*W; 
            end                   
        end
    case 3
        for j=1:length(deltau);
            if  deltau(j)>deltau_max;
                h(end+1,j)=W;
                E(end+1,1)=(deltau_max-tol)*W; 
            end                   
            if  deltau(j)<deltau_min;
                h(end+1,j)=W;
                E(end+1,1)=(deltau_min+tol)*W; 
            end   
        end
end
h=h(2:end,:);
E=E(2:end,1);
%==========================================================================
%     Clculo de las matrices de restricciones h2 y E2 para las entradas
%==========================================================================
function  [E,h]=u_rest(flag,U_fut,U,U_max,U_min,W,tol)
a=length(U_fut);
switch flag
    case 1
        flag2=1;
        h=zeros(1,a);        
        E=(U_min+tol-U)*W;    
        for j=1:a;                
            if U_fut(j)<U_min && flag2==1;
                h(1,1:j)=W;
                flag2=2;
            elseif flag2==2
                h(end+1,j)=W;
            end
        end
    case 2
        flag1=1;
        h=zeros(1,a);        
        E=(U_max-tol-U)*W;    
        for j=1:a;
            if U_fut(j)>U_max && flag1==1;
                h(1,1:j)=W;
                flag1=2;
            elseif flag1==2
                h(end+1,j)=W;
            end   
        end
    case 3
        flag2=1;
        flag1=1;
        h=zeros(2,a);        
        E=[U_max-tol-U;U_min+tol-U]*W;    
        for j=1:a;
            if U_fut(j)>U_max && flag1==1;
                h(1,1:j)=W;
                flag1=2;
            elseif flag1==2
                h(end+1,j)=W;
            end                   
            if U_fut(j)<U_min && flag2==1;
                h(2,1:j)=W;
                flag2=2;
            elseif flag2==2 && h(end,j)==0
                h(end+1,j)=W;
            end
        end
end
E=[E;zeros(size(h,1)-length(E),1)];
%==========================================================================
%Matrices de restricciones h3 y E3 para las predicciones en las salidas
%==========================================================================
function  [E,h]=yp_rest(flag,ypred,y_max,y_min,W,tol,G,L)
h=zeros(1,size(G,2));
E=0;
switch flag
    case 1
        for j=1:length(ypred);                
            if ypred(j)<y_min;
                h(end+1,:)=G(j,:)*W;
                E(end+1,1)=(y_min+tol-L(j))*W;
            end   
        end
    case 2
        for j=1:length(ypred);
            if ypred(j)>y_max;
                h(end+1,:)=G(j,:)*W;
                E(end+1,1)=(y_max-tol-L(j))*W; 
            end                   
        end
    case 3
        for j=1:length(ypred);
            if ypred(j)>y_max;
                h(end+1,:)=G(j,:)*W;
                E(end+1,1)=(y_max-tol-L(j))*W; 
            end                    
            if ypred(j)<y_min;
                h(end+1,:)=G(j,:)*W;
                E(end+1,1)=(y_min+tol-L(j))*W;
            end   
        end
end
h=h(2:end,:);
E=E(2:end,1);
%==========================================================================
%==========================================================================
