clear all; clc
%==========================================================================
%            Obtencin de los coeficientes Su de la rta al escaln
%==========================================================================
Ts=4;%tiempo de muestreo
sis=c2d(tf([0 4.05],[50 1]),Ts,'zoh');%sistema
[numsis densis]=tfdata(sis,'v');
Su=step(sis);
Su=Su(2:end);%vector de coeficientes de respuesta al escaln
N=length(Su);%longitud del vector de coeficientes Su
%==========================================================================
%                               Parmetros
%==========================================================================
instantes=100;%nmero de periodos de muestreo que se ejecutar el algoritmo DMC
Nc=5; %horizonte de control
Np=N+Nc;%horizonte de prediccin
alfa=1;%parmetros alfa lambda y W
lambda=1;
ymin=-0.5;%lmites mximos y mnimos de las VC
ymax= 0.5;
umin=-0.19;%lmites mximos y mnimos y de incrementos de las VM
umax= 0.19;
deltaumin=-0.01;
deltaumax= 0.01;
r=0.55;%referencia 
%==========================================================================
%                Clculo de las matrices G, Q1, Q2, alfa, lambda
%==========================================================================
if Np>N;
    Saux=[Su; ones(Np-N,1)*Su(end)];
else
    Saux=Su;
end
G=zeros(Np,Nc);
for i=1:Nc;
    G(i:end,i)=Saux(1:Np-i+1);
end
Q1=zeros(N,N);
Q1(1:end-1,2:end)=eye(N-1,N-1);
Q1(end,end)=1;
Q2=zeros(Np,N);
if Np==N;
    Q2=Q1;
elseif Np<N
    Q2(1:end,2:Np+1)=eye(Np,Np);
else
    Q2(1:N-1,2:end)=eye(N-1,N-1);
    Q2(N:end,end)=1;
end
alfa=alfa*eye(Np,Np);
lambda=lambda*eye(Nc,Nc);
%==========================================================================
%                       Matrices para el algoritmo QP             
%==========================================================================
H=G'*alfa*G+lambda;
H=(H+H')/2;
I=eye(Nc,Nc);
IL=tril(ones(Nc,Nc));
A1=[I; -I];
A2=[IL; -IL];
A3=[G; -G];
A=[A1; A2; A3];
B1=[ones(Nc,1)*deltaumax; -ones(Nc,1)*deltaumin];
urest=[umax*ones(Nc,1); -umin*ones(Nc,1)];
yrest=[ymax*ones(Np,1); -ymin*ones(Np,1)];
Caux=-G'*alfa;
%==========================================================================
%                    condiciones iniciales
%==========================================================================
t=(0:1:instantes);%vector de tiempo total en instantes
R=r*ones(instantes+Np,1);%vector de trayectoria de referencia
ymvec=0;%Vector que contendra el historial de sallidas medidas
ypvec=ymvec;%vector que contendra el historial de predicciones para cada instante (k+1|k)
deltauvec=0;%vector que contendra el historial de incrementos deltau para cada instante (k|k)
uvec=deltauvec;%vector que contendra el historial de la entrada para cada instante (k|k)
zyu=0;%cond. iniciales para poder simular los sistemas mediante el comando filter
L=ymvec*ones(N,1);%inicializacion de la resuesta libre
%==========================================================================
%                       Ejecucin del algoritmo QDMC
%==========================================================================
for i=1:instantes;
    L=Q1*L+Su*deltauvec(end);%actualizacin de la rta libre
    w=(ymvec(end)-L(1))*ones(Np,1);%estimacin de la perturbacin no medible
    T=Q2*L+w;
    D=R(i:i+Np-1)-T;     
    a=ones(Nc,1)*uvec(end);
    B2=urest+[-a;a];                
    B3=yrest+[-T;T];
    B=[B1; B2; B3];
    C=Caux*D;
    deltau=QUADPROG(H,C,A,B);%clculo del vector de incrementos en la accion de control
    yp=T+G*deltau;%salida predicha
    ypvec(i+1)=yp(1);%salida predicha del instante Yp(k+1|k)
    deltauvec(i+1)=deltau(1);%deltau del instante deltau(k|k) 
    uvec(i+1)=uvec(end)+deltau(1);%accion de control del instante u(k|k)
    %======================================================================
    %       aplicacion de los incrementos y medicin de la salida
    %======================================================================    
    [y,zyu]=filter(numsis(2:end),densis,uvec(end),zyu);%se aplica la accin de control
    ymvec(i+1)=y;%se mide la salida del proceso
    %======================================================================
    %======================================================================
end
R=R(1:instantes+1);
deltauvec=deltauvec(2:end);
uvec=uvec(2:end);
%==========================================================================
%                        Grficas de las variables
%==========================================================================
figure(1)
plot(t,ymvec,'b');
hold on
stairs(t,ypvec,'r');
stairs(t,R,'--g');
hold off
title('Salida')
xlabel('instantes')
legend('Yreal','Ypredicha','Ref')

figure(2)
subplot(211)
stairs(t(1:end-1),uvec,'b');
title('Acciones de control')
xlabel('instantes')
legend('u')
subplot(212)
stairs(t(1:end-1),deltauvec,'b');
title('Incrementos en las acciones de control')
xlabel('instantes')
legend('deltau')